module Strain
  class Variant < ActiveRecord::Base

    has_many :articles, inverse_of: :variant

    validates_presence_of :name
    validates_presence_of :schema
    validate :schema_must_be_valid
    validate :schema_root_must_be_object
    validates :name, uniqueness: { case_sensitive: false }

    private

    def schema_must_be_valid
      meta_schema = JSON.load(
        File.read("#{::Strain::Engine.root}/lib/json_schema/draft_4_meta_schema.json")
      )
      unless JSON::Validator.validate(meta_schema, self.schema)
        errors.add(:schema, "must be a valid Draft 4 JSON Variant.")
      end
    end

    def schema_root_must_be_object
      return if schema.blank?
      #TODO: check for schema w/o 'type' key set at all
      unless schema.fetch("type") == "object"
        errors.add(:schema, "must have an object as root element.")
      end
    end

  end
end
